@@ -1,7 +1,7 @@
 /*
   scsiadd - add/remove scsi devices from the linux scsi subsystem
   ----------------------------------------------------------------
-  Copyright (C) 2000-2007 by Dirk Jagdmann <doj@cubic.org>
+  Copyright (C) 2000..2009 by Dirk Jagdmann <doj@cubic.org>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -201,6 +201,22 @@ static int scsidump(FILE *os)
   return 0;
 }
 
+static int full_atoi(const char *str)
+{
+  char *end = NULL;
+  int ret;
+
+  errno = 0;
+  ret = strtol(str, &end, 10);
+  if(errno || end == str || *end)
+    {
+      fprintf(stderr, "scsiadd: not a valid number: %s\n", str);
+      exit(1);
+    }
+
+  return ret;
+}
+
 /**
    main function.
    @param argc number of command line items
@@ -247,22 +263,22 @@ int main(int argc, char **argv)
       switch(argc-optind)
 	{
 	case 1:
-	  id=atoi(argv[optind]);
+	  id=full_atoi(argv[optind]);
 	  break;
 	case 2:
-	  id=atoi(argv[optind]);
-	  lun=atoi(argv[optind+1]);
+	  id=full_atoi(argv[optind]);
+	  lun=full_atoi(argv[optind+1]);
 	  break;
 	case 3:
-	  host=atoi(argv[optind]);
-	  id=atoi(argv[optind+1]);
-	  lun=atoi(argv[optind+2]);
+	  host=full_atoi(argv[optind]);
+	  id=full_atoi(argv[optind+1]);
+	  lun=full_atoi(argv[optind+2]);
 	  break;
 	case 4:
-	  host=atoi(argv[optind]);
-	  channel=atoi(argv[optind+1]);
-	  id=atoi(argv[optind+2]);
-	  lun=atoi(argv[optind+3]);
+	  host=full_atoi(argv[optind]);
+	  channel=full_atoi(argv[optind+1]);
+	  id=full_atoi(argv[optind+2]);
+	  lun=full_atoi(argv[optind+3]);
 	  break;
 	default:
 	  help();
@@ -272,11 +288,11 @@ int main(int argc, char **argv)
       switch(argc-optind)
 	{
 	case 1:
-	  host=atoi(argv[optind]);
+	  host=full_atoi(argv[optind]);
 	  break;
 	case 2:
-	  host=atoi(argv[optind]);
-	  channel=atoi(argv[optind+1]);
+	  host=full_atoi(argv[optind]);
+	  channel=full_atoi(argv[optind+1]);
 	  break;
 	}
       break;
